function  [Y,BB]  = BK(X,freq_min,freq_max,K)
% PURPOSE:  computes cyclical component of given time series using the
% Baxter-King bandpass filter
%
% INPUT: 
%        X          - time series (T x K)
%        freq_min   - cut-off frequency - number of periods per cycle
%        freq_max   - cut-off frequency - number of periods per cycle
%        K          - order of the filter
%
% OUTPUT: 
%        Y      - cyclical component of X
%
% written by:
% Pawel Kowal
% Department of Economics
% Warsaw School of Economics
% pkowal3@sgh.waw.pl
%
% Modified by Ryan Chahrour (2017)
if K == 0
    Y = X;
    BB = 1;
    return
end

freq_min    = max(2,freq_min);

a           = 2*pi/freq_max;
b           = 2*pi/freq_min;

J           = [1:1:K]';
B           = (sin(b*J)-sin(a*J))./(pi*J);
B           = [(b-a)/pi;B];

%Baxter-King approximation
s           = B(1) + 2*sum(B(2:end));
BB          = B - s/(2*K+1);
BB          = [flipud(BB(2:end));BB];

% Ryan Chahrour: to allow the code to do the first-difference filter.
if freq_min == freq_max
    BB = 0*BB;
    BB(K+1) = 1;
    BB(K) = -1;
end

%filtering
T           = size(X,1);
Y = NaN*X;
for t = K+1:1:T-K
    Y(t,:)  = BB'*X(t-K:t+K,:);    
end
%Y([1:K,end-K+1:end],:)=[];
BB = BB(:)';
